<?php

namespace Drupal\webform_simplify\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\webform\WebformInterface;
use Drupal\webform_simplify\Plugin\WebformSimplifyElement\Defaults;
use Symfony\Component\DependencyInjection\ContainerInterface;

class WebformSimplifySettingsForm extends FormBase {

  /** @var \Drupal\webform_simplify\WebformSimplifyElementManager */
  protected $webformSimplifyElementManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);
    $instance->configFactory = $container->get('config.factory');
    $instance->messenger = $container->get('messenger');
    $instance->webformSimplifyElementManager = $container->get('plugin.manager.webform_simplify.element');

    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'webform_simplify_settings';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $simplifyConfig = $this->configFactory->get('webform_simplify.settings');
    $webformConfig = $this->configFactory->get('webform.settings');

    $form['#attached']['library'][] = 'webform_simplify/settings_form';

    $form['general_settings'] = [
      '#title' => $this->t('General settings'),
      '#type' => 'fieldset',
    ];

    $form['general_settings']['disable_help'] = [
      '#title' => $this->t('Disable help sections'),
      '#type' => 'checkbox',
      '#default_value' => $simplifyConfig->get('disable_help'),
    ];

    $form['element_types'] = [
      '#title' => $this->t('Element types'),
      '#type' => 'fieldset',
    ];

    $excludedElements = $webformConfig->get('element.excluded_elements');
    $elementSettingsUrl = Url::fromRoute('webform.config.elements', [], ['fragment' => 'edit-types'])->toString();

    $form['element_types']['info'] = [
      '#markup' => '<p>' . $this->t("Element types can be excluded so they don't appear as an option when building a webform.") . '<br>'
      . $this->t("You can do this by navigating to <i>Webforms</i> > <i>Configuration</i> > <i>Elements</i> > <i>Element types</i>, or by following <a href=\"@elementSettingsUrl\">this link</a>.", ['@elementSettingsUrl' => $elementSettingsUrl]) . '</p>',
    ];

    $form['element_types']['excluded_elements'] = [
      '#type' => 'details',
      '#title' => $this->t('Excluded elements'),
    ];

    if ($excludedElements === []) {
      $form['element_types']['excluded_elements']['types'] = [
        '#markup' => '<p>' . $this->t('No elements are excluded yet.') . '</p>',
      ];
    } else {
      $form['element_types']['excluded_elements']['types'] = [
        '#markup' => '<p>' . $this->t('The following elements are excluded: <i>@excludedElements</i>.', [
          '@excludedElements' => implode(', ', $excludedElements),
        ]) . '</p>',
      ];
    }

    $form['webform_settings'] = [
      '#title' => $this->t('Webform settings'),
      '#type' => 'fieldset',
      '#tree' => TRUE,
    ];

    $confirmationTypes = [
      WebformInterface::CONFIRMATION_PAGE => $this->t('Page (redirects to new page and displays the confirmation message)'),
      WebformInterface::CONFIRMATION_INLINE => $this->t('Inline (reloads the current page and replaces the webform with the confirmation message)'),
      WebformInterface::CONFIRMATION_MESSAGE => $this->t('Message (reloads the current page/form and displays the confirmation message at the top of the page)'),
      WebformInterface::CONFIRMATION_MODAL => $this->t('Modal (reloads the current page/form and displays the confirmation message in a modal dialog)'),
      WebformInterface::CONFIRMATION_URL => $this->t('URL (redirects to a custom path or URL)'),
      WebformInterface::CONFIRMATION_URL_MESSAGE => $this->t('URL with message (redirects to a custom path or URL and displays the confirmation message at the top of the page)'),
      WebformInterface::CONFIRMATION_NONE => $this->t('None (reloads the current page and does not display a confirmation message)'),
    ];

    $disabledConfirmationTypes = $simplifyConfig->get('webform_settings.disabled_confirmation_types') ?? [];
    $enabledConfirmationTypes = array_diff(array_keys($confirmationTypes), $disabledConfirmationTypes);

    $form['webform_settings']['confirmation_types'] = [
      '#title' => $this->t('Confirmation types'),
      '#type' => 'checkboxes',
      '#options' => $confirmationTypes,
      '#default_value' => $enabledConfirmationTypes,
    ];

    $form['element_settings'] = [
      '#title' => $this->t('Element settings'),
      '#type' => 'fieldset',
      '#tree' => TRUE,
    ];

    $definitions = $this->webformSimplifyElementManager->getDefinitions();

    // Sort the definitions by label.
    uasort($definitions, function ($a, $b) {
      return strcasecmp($a['label'], $b['label']);
    });

    // Move the defaults to the top.
    $definitions = [Defaults::ID => $definitions[Defaults::ID]] + $definitions;

    foreach ($definitions as $id => $definition) {
      if (in_array($id, $excludedElements)) {
        continue;
      }

      /** @var \Drupal\webform_simplify\WebformSimplifyElementInterface $simplifyElement */
      $simplifyElement = $this->webformSimplifyElementManager->createInstance($id);

      $form['element_settings'][$id] = [
        '#title' => $definition['label'],
        '#type' => 'details',
        '#tree' => TRUE,
        '#open' => FALSE,
      ];

      $tabs = $simplifyElement->getTabs();
      $disabledTabs = $this->getElementSetting($id, 'disabled_tabs') ?? [];
      $enabledTabs = array_diff(array_keys($tabs), $disabledTabs);

      $form['element_settings'][$id]['tabs'] = [
        '#title' => $this->t('Tabs'),
        '#type' => 'checkboxes',
        '#options' => $tabs,
        '#default_value' => $enabledTabs,
      ];

      $features = $simplifyElement->getFeatures();
      $disabledFeatures = $this->getElementSetting($id, 'disabled_features') ?? [];
      $enabledFeatures = array_diff(array_keys($features), $disabledFeatures);

      $form['element_settings'][$id]['features'] = [
        '#title' => $this->t('Features'),
        '#type' => 'checkboxes',
        '#options' => $features,
        '#default_value' => $enabledFeatures,
      ];
    }

    $form['actions']['#type'] = 'actions';
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save configuration'),
      '#button_type' => 'primary',
    ];

    // By default, render the form using system-config-form.html.twig.
    $form['#theme'] = 'system_config_form';

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $config = $this->configFactory->getEditable('webform_simplify.settings');
    $values = $form_state->cleanValues()->getValues();

    $config->set('disable_help', $values['disable_help']);

    $disabledConfirmationTypes = array_filter(
      $values['webform_settings']['confirmation_types'],
      function ($value) {
        return $value === 0;
      }
    );
    $config->set('webform_settings.disabled_confirmation_types', array_keys($disabledConfirmationTypes));

    foreach ($values['element_settings'] as $element => $settings) {
      $disabledTabs = array_filter(
            $settings['tabs'],
            function ($value) {
              return $value === 0;
            }
        );

      $config->set(
            sprintf('element_settings.%s.disabled_tabs', $element),
            array_keys($disabledTabs)
        );

      $disabledFeatures = array_filter(
            $settings['features'],
            function ($value) {
              return $value === 0;
            }
        );

      $config->set(
            sprintf('element_settings.%s.disabled_features', $element),
            array_keys($disabledFeatures)
        );
    }

    // Clean up config.
    $elementSettings = $config->get('element_settings');
    $defaultSettings = $elementSettings[Defaults::ID] ?? [];

    foreach ($elementSettings as $element => &$settings) {
      if (($settings['disabled_tabs'] ?? []) === [] && ($settings['disabled_features'] ?? []) === []) {
        unset($elementSettings[$element]);
      }

      // If the settings are the same as the defaults, remove them (except for the defaults themselves).
      if ($element !== Defaults::ID) {
        foreach (['disabled_tabs', 'disabled_features'] as $settingKey) {
          if (array_diff($settings[$settingKey], $defaultSettings[$settingKey]) === []) {
            unset($settings[$settingKey]);
          }
        }
        if ($settings === []) {
          unset($elementSettings[$element]);
        }
      }
    }

    $config->set('element_settings', $elementSettings);
    $config->save();
    $this->messenger->addStatus($this->t('The configuration options have been saved.'));
  }

  /**
   * Get an element setting, with a fallback to the default settings.
   *
   * @param string $elementKey
   *   The element key.
   * @param string $settingKey
   *   The setting key.
   *
   * @return mixed
   *   The setting value.
   */
  protected function getElementSetting(string $elementKey, string $settingKey) {
    $simplifyConfig = $this->configFactory->get('webform_simplify.settings');

    return $simplifyConfig->get(sprintf('element_settings.%s.%s', $elementKey, $settingKey))
      ?? $simplifyConfig->get(sprintf('element_settings.%s.%s', Defaults::ID, $settingKey));
  }

}
